#!/usr/bin/python
# coding=utf-8


from bcc import BPF
import socket
import os
from struct import *


# Programma eBPF #
bpf_program = """\

	#include <bcc/proto.h>

	int filter (struct __sk_buff *skb) {
		struct ethernet_t *ether_header = 0;
		struct ip_t *ip_header = (struct ip_t *) sizeof (struct ethernet_t);
		struct tcp_t *tcp_header = (struct tcp_t *) (sizeof (struct ethernet_t) + sizeof (struct ip_t));

		/* Ignora il Pacchetto se Non è IPv4 */
		if (ether_header->type != 0x0800)
			return 0;

		/* Ignora il Pacchetto se Non è TCP */
		if (ip_header->nextp != 0x06)
			return 0;

		/* Ignora il Pacchetto se Non è Diretto/Ricevuto verso/da l'indirizzo IP Scelto (130.192.181.193) */
		if (ip_header->src != 0x82c0b5c1 && ip_header->dst != 0x82c0b5c1)
			return 0;

		/* Invia il Pacchetto a Livello Utente */
		return -1;
	}

"""


# Inizializza eBPF #
bpf = BPF (text = bpf_program)

# Attacca il Programma eBPF a un Socket di Livello Kernel #
filter_func = bpf.load_func ("filter", BPF.SOCKET_FILTER)
BPF.attach_raw_socket (filter_func, "eth0")

# Inizializza il Socket di Livello Utente #
sock_fd = filter_func.sock
user_socket = socket.fromfd (sock_fd, socket.PF_PACKET, socket.SOCK_RAW, socket.IPPROTO_IP)
user_socket.setblocking (True)

# Loop Principale #
ether_header_len = 14
print ("\n[*] Premere ctrl+c per Terminare...\n")
while 1:
	try:

		# Leggi il Pacchetto
		packet = os.read (sock_fd, 2048)

		# Traduci i Byte del Pacchetto negli Header IP-TCP
		ip_header = unpack ('!BBHHHBBH4s4s', packet[ether_header_len:ether_header_len+20])
		ip_header_len = (ip_header[0] & 0xF) * 4
		tcp_header = unpack ('!HHLLBBHHH', packet[ether_header_len+ip_header_len:ether_header_len+ip_header_len+20])

		# Stampa Informazioni sul Pacchetto
		src_ip = socket.inet_ntoa (ip_header[8]);
		dst_ip = socket.inet_ntoa (ip_header[9]);
		print ("[*] %s:%d -> %s:%d (S:%d P:%d A:%d F:%d)" % (str(src_ip), tcp_header[0], str(dst_ip), tcp_header[1], \
									(lambda : 1 if (tcp_header[5] & 2) > 0 else 0)(), \
									(lambda : 1 if (tcp_header[5] & 8) > 0 else 0)(), \
									(lambda : 1 if (tcp_header[5] & 16) > 0 else 0)(), \
									(lambda : 1 if (tcp_header[5] & 1) > 0 else 0)()))

	except KeyboardInterrupt:

		# Termina Monitoraggio #
		break
print ("\n")
